@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: script.py 3890 2008-05-09 17:51:18Z richb $"
-__version__   = "$Revision: 3890 $"
-__date__      = "$Date: 2008-05-09 13:51:18 -0400 (Fri, 09 May 2008) $"
+__id__        = "$Id: script.py 3974 2008-06-11 21:48:33Z richb $"
+__version__   = "$Revision: 3974 $"
+__date__      = "$Date: 2008-06-11 17:48:33 -0400 (Wed, 11 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -29,7 +29,6 @@ import gtk
 import pyatspi
 
 import orca.debug as debug
-import orca.chnames as chnames
 import orca.default as default
 import orca.input_event as input_event
 import orca.braille as braille
@@ -44,7 +43,7 @@ from orca.orca_i18n import _ # for gettext support
 from speech_generator import SpeechGenerator
 from braille_generator import BrailleGenerator
 from where_am_i import WhereAmI
-from constants import speakCellCoordinates
+import script_settings
 
 class Script(default.Script):
 
@@ -232,39 +231,39 @@ class Script(default.Script):
         keyBindings.add(
             keybindings.KeyBinding(
                 "a",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["speakInputLineHandler"]))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "r",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["setDynamicColumnHeadersHandler"],
                 1))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "r",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["clearDynamicColumnHeadersHandler"],
                 2))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "c",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["setDynamicRowHeadersHandler"],
                 1))
 
         keyBindings.add(
             keybindings.KeyBinding(
                 "c",
-                1 << settings.MODIFIER_ORCA,
-                1 << settings.MODIFIER_ORCA,
+                settings.defaultModifierMask,
+                settings.ORCA_MODIFIER_MASK,
                 self.inputEventHandlers["clearDynamicRowHeadersHandler"],
                 2))
 
@@ -291,7 +290,7 @@ class Script(default.Script):
         gtk.Box.pack_start(vbox, self.speakCellCoordinatesCheckButton,
                            False, False, 0)
         gtk.ToggleButton.set_active(self.speakCellCoordinatesCheckButton,
-                                    speakCellCoordinates)
+                                    script_settings.speakCellCoordinates)
 
         return vbox
 
@@ -303,17 +302,12 @@ class Script(default.Script):
         - prefs: file handle for application preferences.
         """
 
-        global speakCellCoordinates
-
         prefs.writelines("\n")
-        speakCellCoordinates = \
+        script_settings.speakCellCoordinates = \
                  self.speakCellCoordinatesCheckButton.get_active()
-        prefix = "orca.scripts.apps.soffice.script"
+        prefix = "orca.scripts.apps.soffice.script_settings"
         prefs.writelines("%s.speakCellCoordinates = %s\n" % \
-                         (prefix, speakCellCoordinates))
-        prefix = "orca.scripts.apps.soffice.speech_generator"
-        prefs.writelines("%s.speakCellCoordinates = %s\n" % \
-                         (prefix, speakCellCoordinates))
+                         (prefix, script_settings.speakCellCoordinates))
 
     def getAppState(self):
         """Returns an object that can be passed to setAppState.  This
@@ -395,7 +389,8 @@ class Script(default.Script):
             parentTable = None
 
         if parent and parentTable:
-            row = parentTable.getRowAtIndex(obj.getIndexInParent())
+            index = self.getCellIndex(obj)
+            row = parentTable.getRowAtIndex(index)
             accCell = parentTable.getAccessibleAt(row, column)
 
         return accCell
@@ -421,7 +416,8 @@ class Script(default.Script):
             parentTable = None
 
         if parent and parentTable:
-            column = parentTable.getColumnAtIndex(obj.getIndexInParent())
+            index = self.getCellIndex(obj)
+            column = parentTable.getColumnAtIndex(index)
             accCell = parentTable.getAccessibleAt(row, column)
 
         return accCell
@@ -483,14 +479,16 @@ class Script(default.Script):
                 parent.queryComponent().getAccessibleAtPoint(leftX, y, 0)
             if leftCell:
                 table = leftCell.parent.queryTable()
-                startIndex = table.getColumnAtIndex(leftCell.getIndexInParent())
+                index = self.getCellIndex(leftCell)
+                startIndex = table.getColumnAtIndex(index)
 
             rightX = extents.x + extents.width - 1
             rightCell = \
                 parent.queryComponent().getAccessibleAtPoint(rightX, y, 0)
             if rightCell:
                 table = rightCell.parent.queryTable()
-                endIndex = table.getColumnAtIndex(rightCell.getIndexInParent())
+                index = self.getCellIndex(rightCell)
+                endIndex = table.getColumnAtIndex(index)
 
         return [startIndex, endIndex]
 
@@ -536,7 +534,7 @@ class Script(default.Script):
                     found = True
                 current = current.parent
 
-        return found     
+        return found
 
     def isDesiredFocusedItem(self, obj, rolesList):
         """Called to determine if the given object and it's hierarchy of
@@ -564,10 +562,10 @@ class Script(default.Script):
 
             if current_role != role:
                 return False
-            
+
             current = self._getParent(current)
 
-        return True   
+        return True
 
     def printHierarchy(self, root, ooi, indent="",
                        onlyShowing=True, omitManaged=True):
@@ -629,14 +627,13 @@ class Script(default.Script):
             for frame in app:
                 if frame.childCount < 1 or \
                       frame[0].getRole() not in (pyatspi.ROLE_ROOT_PANE,
-                                                 pyatspi.ROLE_OPTION_PANE): 
+                                                 pyatspi.ROLE_OPTION_PANE):
                     continue
                 root_pane = frame[0]
                 if obj in root_pane:
                     return root_pane
         return parent
 
-
     def checkForTableBoundry(self, oldFocus, newFocus):
         """Check to see if we've entered or left a table.
         When entering a table, announce the table dimensions.
@@ -727,7 +724,8 @@ class Script(default.Script):
                 parentTable = None
 
             if parent and parentTable:
-                row = parentTable.getRowAtIndex(cell.getIndexInParent())
+                index = self.getCellIndex(cell)
+                row = parentTable.getRowAtIndex(index)
 
         return row
 
@@ -751,7 +749,8 @@ class Script(default.Script):
                 parentTable = None
 
             if parent and parentTable:
-                column = parentTable.getColumnAtIndex(cell.getIndexInParent())
+                index = self.getCellIndex(cell)
+                column = parentTable.getColumnAtIndex(index)
 
         return column
 
@@ -1061,7 +1060,7 @@ class Script(default.Script):
 
     def __isAvailableFieldsPanel(self, event):
         """If we are in the sbase Table Wizard, try to reduce the numerous
-        utterances of "Available fields panel". See bug #465087 for more 
+        utterances of "Available fields panel". See bug #465087 for more
         details.
 
         Arguments:
@@ -1111,6 +1110,53 @@ class Script(default.Script):
 
         return isPanel
 
+    def _speakWriterText(self, event, textToSpeak):
+        """Called to speak the current line or paragraph of Writer text.
+
+        Arguments:
+        - event: the Event
+        - textToSpeak: the text to speak
+        """
+
+        # Check to see if there are any hypertext links in this paragraph.
+        # If no, then just speak the whole line. Otherwise, split the text
+        # to speak into words and call sayWriterWord() to speak that token
+        # in the appropriate voice.
+        #
+        try:
+            hypertext = event.source.queryHypertext()
+        except NotImplementedError:
+            hypertext = None
+
+        if not hypertext or (hypertext.getNLinks() == 0):
+            if settings.enableSpeechIndentation:
+                self.speakTextIndentation(event.source,
+                                          textToSpeak.encode("UTF-8"))
+            speech.speak(textToSpeak.encode("UTF-8"), None, False)
+        else:
+            started = False
+            startOffset = 0
+            for i in range(0, len(textToSpeak)):
+                if textToSpeak[i] == ' ':
+                    if started:
+                        endOffset = i
+                        self.sayWriterWord(event.source,
+                            textToSpeak[startOffset:endOffset+1].encode( \
+                                                                "UTF-8"),
+                            startOffset, endOffset)
+                        startOffset = i
+                        started = False
+                else:
+                    if not started:
+                        startOffset = i
+                        started = True
+
+            if started:
+                endOffset = len(textToSpeak)
+                self.sayWriterWord(event.source,
+                    textToSpeak[startOffset:endOffset].encode("UTF-8"),
+                    startOffset, endOffset)
+
     # This method tries to detect and handle the following cases:
     # 0) Writer: find command.
     # 1) Writer: text paragraph.
@@ -1172,7 +1218,7 @@ class Script(default.Script):
                   "StarOffice.locusOfFocusChanged - Writer: text paragraph.")
 
             result = self.getTextLineAtCaret(event.source)
-            result[0] = result[0].decode("UTF-8")
+            textToSpeak = result[0].decode("UTF-8")
 
             # Translators: this is the name of the menu item people
             # use in StarOffice to create a new text document.  It's
@@ -1184,50 +1230,11 @@ class Script(default.Script):
             if oldLocusOfFocus and \
                oldLocusOfFocus.getRole() == pyatspi.ROLE_MENU_ITEM and \
                oldLocusOfFocus.name == _("Text Document") and \
-               len(result[0]) == 0:
+               len(textToSpeak) == 0:
                 self.whereAmI.whereAmI(None)
 
-            # Check to see if there are any hypertext links in this paragraph.
-            # If no, then just speak the whole line. Otherwise, split the
-            # line into words and call sayWriterWord() to speak that token
-            # in the appropriate voice.
-            #
-            try:
-                hypertext = event.source.queryHypertext()
-            except NotImplementedError:
-                hypertext = None
-
-            if not hypertext or (hypertext.getNLinks() == 0):
-                if settings.enableSpeechIndentation:
-                    self.speakTextIndentation(event.source,
-                                              result[0].encode("UTF-8"))
-                speech.speak(result[0].encode("UTF-8"), None, False)
-            else:
-                started = False
-                startOffset = 0
-                for i in range(0, len(result[0])):
-                    if result[0][i] == ' ':
-                        if started:
-                            endOffset = i
-                            self.sayWriterWord(event.source,
-                                result[0][startOffset:endOffset+1].encode( \
-                                                                    "UTF-8"),
-                                startOffset, endOffset)
-                            startOffset = i
-                            started = False
-                    else:
-                        if not started:
-                            startOffset = i
-                            started = True
-
-                if started:
-                    endOffset = len(result[0])
-                    self.sayWriterWord(event.source,
-                        result[0][startOffset:endOffset].encode("UTF-8"),
-                        startOffset, endOffset)
-
+            self._speakWriterText(event, textToSpeak)
             braille.displayRegions(brailleGen.getBrailleRegions(event.source))
-
             return
 
         # 2) Writer: spell checking dialog.
@@ -1430,11 +1437,10 @@ class Script(default.Script):
                 except:
                     pass
                 else:
-                    column = table.getColumnAtIndex( \
-                                    newLocusOfFocus.getIndexInParent())
+                    index = self.getCellIndex(newLocusOfFocus)
+                    column = table.getColumnAtIndex(index)
                     self.pointOfReference['lastColumn'] = column
-                    row = table.getRowAtIndex( \
-                                    newLocusOfFocus.getIndexInParent())
+                    row = table.getRowAtIndex(index)
                     self.pointOfReference['lastRow'] = row
                 return
 
@@ -1482,7 +1488,6 @@ class Script(default.Script):
             #
             default.Script.onWindowActivated(self, event)
 
-
     # This method tries to detect and handle the following cases:
     # 1) Writer: spell checking dialog.
 
@@ -1536,7 +1541,6 @@ class Script(default.Script):
 
         default.Script.onNameChanged(self, event)
 
-
     def onFocus(self, event):
         """Called whenever an object gets focus.
 
@@ -1544,7 +1548,7 @@ class Script(default.Script):
         - event: the Event
         """
 
-        # If this is a "focus:" event for the Calc Name combo box, catch 
+        # If this is a "focus:" event for the Calc Name combo box, catch
         # it here to reduce verbosity (see bug #364407).
         #
         rolesList = [pyatspi.ROLE_LIST, \
@@ -1606,7 +1610,7 @@ class Script(default.Script):
                 if current.getRole() == pyatspi.ROLE_DIALOG and \
                    (current.name and \
                     current.name.startswith(_("Presentation Wizard"))):
-                    self.locusOfFocusChanged(event, None, 
+                    self.locusOfFocusChanged(event, None,
                                              orca_state.locusOfFocus)
                     break
                 current = current.parent
@@ -1657,11 +1661,7 @@ class Script(default.Script):
                 keyString = orca_state.lastNonModifierKeyEvent.event_string
                 navKeys = ["Up", "Down", "Page_Up", "Page_Down", "Home", "End"]
                 wasCommand = orca_state.lastInputEvent.modifiers \
-                             & (1 << pyatspi.MODIFIER_CONTROL \
-                              | 1 << pyatspi.MODIFIER_ALT \
-                              | 1 << pyatspi.MODIFIER_META \
-                              | 1 << pyatspi.MODIFIER_META2 \
-                              | 1 << pyatspi.MODIFIER_META3)
+                             & settings.COMMAND_MODIFIER_MASK
                 weToggledIt = wasCommand and keyString not in navKeys
 
             if weToggledIt:
@@ -1729,7 +1729,6 @@ class Script(default.Script):
 
         default.Script.onStateChanged(self, event)
 
-
     # This method tries to detect and handle the following cases:
     # 1) Calc: spread sheet Name Box line.
 
@@ -1785,7 +1784,7 @@ class Script(default.Script):
                                                  self.inputLineForCell, 0, -1)
                                         if inputLine and (len(inputLine) > 1) \
                                             and (inputLine[0] == "="):
-                                            # Translators: this means a 
+                                            # Translators: this means a
                                             # particular cell in a spreadsheet
                                             # has a formula
                                             # (e.g., "=sum(a1:d1)")
@@ -1853,13 +1852,20 @@ class Script(default.Script):
         - event: the Event
         """
 
-        if orca_state.lastNonModifierKeyEvent:
+        # If the object is losing focus, then just ignore this event.
+        #
+        if event.detail1 == -1:
+            return
+
+        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
+           and orca_state.lastNonModifierKeyEvent:
             event_string = orca_state.lastNonModifierKeyEvent.event_string
-            isShiftKey = orca_state.lastNonModifierKeyEvent.modifiers \
-                         & (1 << pyatspi.MODIFIER_SHIFT)
+            mods = orca_state.lastInputEvent.modifiers
         else:
             event_string = ''
-            isShiftKey = False
+            mods = 0
+        isControlKey = mods & settings.CTRL_MODIFIER_MASK
+        isShiftKey = mods & settings.SHIFT_MODIFIER_MASK
 
         # If we are FOCUSED on a paragraph inside a table cell (in Writer),
         # then just return (modulo the special cases below). Speaking and
@@ -1926,10 +1932,6 @@ class Script(default.Script):
         if (event_string == "Up" or event_string == "Down") and not isShiftKey:
             speech.stop()
 
-        # Speak a newline, if appropriate.
-        if self.speakNewLine(event.source):
-            speech.speak(chnames.getCharacterName("\n"), None, False)
-
         # Speak a blank line, if appropriate.
         if self.speakBlankLine(event.source):
             # Translators: "blank" is a short word to mean the
@@ -1937,66 +1939,19 @@ class Script(default.Script):
             #
             speech.speak(_("blank"), None, False)
 
-        default.Script.onCaretMoved(self, event)
-
-
-    def speakNewLine(self, obj):
-        """Returns True if a newline should be spoken.
-           Otherwise, returns False.
-        """
-
-        # Get the the AccessibleText interface.
-        try:
-            text = obj.queryText()
-        except NotImplementedError:
-            return False
-
-        # Was a left or right-arrow key pressed?
-        if not (orca_state.lastInputEvent and \
-                orca_state.lastInputEvent.__dict__.has_key("event_string")):
-            return False
-
-        lastKey = orca_state.lastNonModifierKeyEvent.event_string
-        if lastKey != "Left" and lastKey != "Right":
-            return False
-
-        # Was a control key pressed?
-        mods = orca_state.lastInputEvent.modifiers
-        isControlKey = mods & (1 << pyatspi.MODIFIER_CONTROL)
-
-        # Get the line containing the caret
-        caretOffset = text.caretOffset
-        line = text.getTextAtOffset(caretOffset, \
-            pyatspi.TEXT_BOUNDARY_LINE_START)
-        lineStart = line[1]
-        lineEnd = line[2]
-
-        if isControlKey:  # control-right-arrow or control-left-arrow
-
-            # Get the word containing the caret.
-            word = text.getTextAtOffset(caretOffset, \
-                pyatspi.TEXT_BOUNDARY_WORD_START)
-            wordStart = word[1]
-            wordEnd = word[2]
-
-            if lastKey == "Right":
-                if wordStart == lineStart:
-                    return True
-            else:
-                if wordEnd == lineEnd:
-                    return True
-
-        else:  # right arrow or left arrow
-
-            if lastKey == "Right":
-                if caretOffset == lineStart:
-                    return True
-            else:
-                if caretOffset == lineEnd:
-                    return True
-
-        return False
-
+        # If the last input event was a keyboard event of Control-Up or
+        # Control-Down, we want to speak the whole paragraph rather than
+        # just the current line.
+        #
+        if (event_string == "Up" or event_string == "Down") and \
+            isControlKey and not isShiftKey:
+            result = self.getText(event.source, 0, -1)
+            textToSpeak = result.decode("UTF-8")
+            self._speakWriterText(event, textToSpeak)
+            braille.displayRegions( \
+                self.brailleGenerator.getBrailleRegions(event.source))
+        else:
+            default.Script.onCaretMoved(self, event)
 
     def speakBlankLine(self, obj):
         """Returns True if a blank line should be spoken.
@@ -2050,9 +2005,9 @@ class Script(default.Script):
         - acc: An accessible.
         - offset: Offset in the accessible's text for which to retrieve the
         attributes.
-        - get_defaults: Get the default attributes as well as the unique ones. 
+        - get_defaults: Get the default attributes as well as the unique ones.
         Default is True
-        
+
         Returns a dictionary of attributes, a start offset where the attributes
         begin, and an end offset. Returns ({}, 0, 0) if the accessible does not
         supprt the text attribute.
@@ -2060,3 +2015,33 @@ class Script(default.Script):
         rv, start, end = \
             default.Script.getTextAttributes(self, acc, offset, get_defaults)
         return rv, start, end - 1
+
+    def isFunctionalDialog(self, obj):
+        """Returns true if the window is functioning as a dialog."""
+
+        # The OOo Navigator window looks like a dialog, acts like a
+        # dialog, and loses focus requiring the user to know that it's
+        # there and needs Alt+F6ing into.  But officially it's a normal
+        # window.
+ 
+        # There doesn't seem to be (an efficient) top-down equivalent
+        # of isDesiredFocusedItem(). But OOo documents have root panes;
+        # this thing does not.
+        #
+        rolesList = [pyatspi.ROLE_FRAME,
+                     pyatspi.ROLE_PANEL,
+                     pyatspi.ROLE_PANEL,
+                     pyatspi.ROLE_TOOL_BAR,
+                     pyatspi.ROLE_PUSH_BUTTON]
+
+        if obj.getRole() != rolesList[0]:
+            # We might be looking at the child.
+            #
+            rolesList.pop(0)
+
+        while obj and obj.childCount and len(rolesList):
+            if obj.getRole() != rolesList.pop(0):
+                return False
+            obj = obj[0]
+
+        return True
