@@ -21,14 +21,17 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id: settings.py 3971 2008-06-11 00:54:46Z joanied $"
+__version__   = "$Revision: 3971 $"
+__date__      = "$Date: 2008-06-10 20:54:46 -0400 (Tue, 10 Jun 2008) $"
 __copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
 import re
+import shutil
+
+import platform
 
 screenWidth = 640
 screenHeight = 480
@@ -77,6 +80,8 @@ try:
 except:
     gconfClient = None
 
+import pyatspi
+
 import debug
 from acss import ACSS
 from orca_i18n import _           # for gettext support
@@ -694,6 +699,14 @@ useBonoboMain           = True
 #
 asyncMode               = True
 
+# A list of toolkits whose events we need to process synchronously.
+# The only one right now is the Java toolkit (see bug #531869), but
+# we put this here to allow more toolkits to be more easily added
+# and for Java to be removed if issues with asynchronous processing
+# of its events are ever resolved.
+#
+synchronousToolkits     = ["J2SE-access-bridge"]
+
 # If True, we output debug information for the event queue.  We
 # use this in addition to log level to prevent debug logic from
 # bogging down event handling.
@@ -738,6 +751,54 @@ def setAccessibilityEnabled(enable):
     except:
         return False
 
+# Obtain/set information regarding whether Orca is autostarted for this 
+# user at login time.
+#
+def isOrcaAutostarted():
+    """Return an indication of whether Orca autostart at login time is enabled.
+    """
+
+    userDesktopFile = os.path.join(os.environ["HOME"], ".config", \
+                                   "autostart", "orca.desktop")
+    return os.path.exists(userDesktopFile)
+
+def setOrcaAutostart(enable):
+    """Enable or disable the autostart of Orca at login time.
+
+    Arguments:
+    - enable: if True, whether Orca autostart at login time is enabled.
+
+    Returns an indication of whether the operation was successful.
+    """
+
+    status = True
+    systemDesktopFile = os.path.join(platform.prefix, platform.datadirname, \
+                                     "applications", "orca.desktop")
+    autostartDir = os.path.join(os.environ["HOME"], ".config", "autostart")
+    userDesktopFile = os.path.join(os.environ["HOME"], ".config", \
+                                   "autostart", "orca.desktop")
+
+    try:
+        # If the user wants to enable Orca autostart, then copy the
+        # orca.desktop file from /usr/share/applications to
+        # ~/.config/autostart directory (creating the autostart directory
+        # if neccessary).
+        #
+        # If the user wants to disable this feature, then just remove 
+        # ~/.config/autostart/orca.desktop (if it exists).
+        #
+        if enable:
+            try:
+                os.chdir(autostartDir)
+            except:
+                os.mkdir(autostartDir)
+            shutil.copy(systemDesktopFile, userDesktopFile)
+        else:
+            os.remove(userDesktopFile)
+    except:
+        status = False
+    return status
+
 # Obtain/set information regarding whether the gksu keyboard grab is enabled
 # or not.
 #
@@ -947,3 +1008,47 @@ mouseDwellDelay = 0
 # Maximum allowed drift while pointer is dwelling in mouse review mode.
 # 
 mouseDwellMaxDrift = 3
+
+# The different modifiers/modifier masks associated with keybindings
+#
+NO_MODIFIER_MASK              =  0
+ALT_MODIFIER_MASK             =  1 << pyatspi.MODIFIER_ALT
+CTRL_MODIFIER_MASK            =  1 << pyatspi.MODIFIER_CONTROL
+ORCA_MODIFIER_MASK            =  1 << MODIFIER_ORCA
+ORCA_ALT_MODIFIER_MASK        = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_ALT)
+ORCA_CTRL_MODIFIER_MASK       = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_CONTROL)
+ORCA_CTRL_ALT_MODIFIER_MASK   = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_ALT)
+ORCA_SHIFT_MODIFIER_MASK      = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_SHIFT)
+SHIFT_MODIFIER_MASK           =  1 << pyatspi.MODIFIER_SHIFT
+SHIFT_ALT_MODIFIER_MASK       = (1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_ALT)
+COMMAND_MODIFIER_MASK         = (1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3)
+NON_LOCKING_MODIFIER_MASK     = (1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3 |
+                                 1 << MODIFIER_ORCA)
+ALL_BUT_NUMLOCK_MODIFIER_MASK = (1 << MODIFIER_ORCA |
+                                 1 << pyatspi.MODIFIER_SHIFT |
+                                 1 << pyatspi.MODIFIER_SHIFTLOCK |
+                                 1 << pyatspi.MODIFIER_CONTROL |
+                                 1 << pyatspi.MODIFIER_ALT |
+                                 1 << pyatspi.MODIFIER_META2 |
+                                 1 << pyatspi.MODIFIER_META3)
+
+# The 2nd parameter used when creating a keybinding refers to the
+# modifiers "we care about."  We typically care about all of the
+# modifiers which are not locking keys because we want to avoid
+# conflicts with other commands, such as Orca's command for moving
+# among headings (H) and the Help menu (Alt+H).
+#
+defaultModifierMask = NON_LOCKING_MODIFIER_MASK
