@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3910 2008-05-20 20:40:55Z joanied $"
-__version__   = "$Revision: 3910 $"
-__date__      = "$Date: 2008-05-20 16:40:55 -0400 (Tue, 20 May 2008) $"
+__id__        = "$Id: speechgenerator.py 3957 2008-06-04 16:43:04Z richb $"
+__version__   = "$Revision: 3957 $"
+__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1466,8 +1466,8 @@ class SpeechGenerator:
                 # the same row).
                 #
                 speakAll = True
-                if self._script.pointOfReference.has_key("lastRow") and \
-                    self._script.pointOfReference.has_key("lastColumn"):
+                if "lastRow" in self._script.pointOfReference and \
+                    "lastColumn" in self._script.pointOfReference:
                     pointOfReference = self._script.pointOfReference
                     speakAll = (pointOfReference["lastRow"] != row) or \
                         ((row == 0 or row == parent_table.nRows-1) and \
@@ -1746,7 +1746,7 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken.
         """
         role = obj.getRole()
-        if self.speechGenerators.has_key(role):
+        if role in self.speechGenerators:
             generator = self.speechGenerators[role]
         else:
             generator = self._getDefaultSpeech
