@@ -19,9 +19,9 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 3910 2008-05-20 20:40:55Z joanied $"
-__version__   = "$Revision: 3910 $"
-__date__      = "$Date: 2008-05-20 16:40:55 -0400 (Tue, 20 May 2008) $"
+__id__        = "$Id: where_am_I.py 3957 2008-06-04 16:43:04Z richb $"
+__version__   = "$Revision: 3957 $"
+__date__      = "$Date: 2008-06-04 12:43:04 -0400 (Wed, 04 Jun 2008) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -1536,7 +1536,7 @@ class WhereAmI:
                           "charDict: %s" % (charDict))
 
             for key in keys:
-                if charDict.has_key(key):
+                if key in charDict:
                     attribute = charDict[key]
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
